/* config.h.  Generated from config.h.in by configure.  */
/* config.h.in.  Generated from configure.in by autoheader.  */
/* $Id */
/* Name of package */
#define PACKAGE "shotdetect"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "johan.mathe@tremplin-utc.fr"

/* Define to the full name of this package. */
#define PACKAGE_NAME "film.cpp"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "film.cpp 1.0"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "film-cpp"

/* Define to the version of this package. */
#define PACKAGE_VERSION "1.0"

/* Version number of package */
#define VERSION "1.0.1"
